#!/usr/bin/perl -wT

=head1 NAME

cmconfig.pl

=cut

=head1 SYNOPSIS

Graphical comicman configuration tool.

=cut

=head1 DESCRIPTION

cmconfig.pl is a graphical (PerlTk) configuration tool for the ComicMan webcomics downloader. 
It handles initial setup as well as editing the comic download list and setting options for the user.

=cut

=head1 AUTHOR

Jay Scott Raymond

=cut

=head1 BUGS

None known as yet.

=cut

=head1 SEE ALSO

inf-doc.txt

=cut

=head1 COPYRIGHT

Copyright 2006, Jay Scott Raymond. All Rights Reserved

This program is free software. You may copy or
redistribute it under the same terms as Perl itself.

=cut

use utf8;
use strict;
use warnings;
use lib '/usr/share/comicman/pm';

use subs qw/build_selector_form build_menubar init/; 
use vars qw/$main_window $main_widget $comic_selector $settings $page1 $page2 $VERSION %inf_comic $listbox_available 
            $listbox_current @list_cur_var @list_avail_var/;

$ENV{'PATH'}="/usr/bin:/bin";
$ENV{SHELL} = '/bin/sh' if exists $ENV{SHELL};
delete @ENV{qw(IFS CDPATH ENV BASH_ENV)};   # Make %ENV safer

package main;

use Tk;
use Tk::widgets qw/Dialog/;
use Tk::NoteBook;
use Comiclib;
use cmComicSelector;
use cmSettingsWidget;

#
# Global variables
#
my $main_window;
my $VERSION = "0.0.3";
my %inf_comic;
my $listbox_available;
my $listbox_current;
my @list_cur_var;
my @list_avail_var;
my $current_log;
my $current_debug;
my $current_timeout;
my $main_widget;
my $page1;
my $page2;
my $comic_selector;
my $settings;


#################################################################
# Subroutine that builds the menu objects
# Params: none
# Returns: none
#################################################################
sub build_menubar() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "build_menus:got $num_params params, incorrect\n";
}

my $menu_bar = $main_window->Menu;
$main_window->configure(-menu => $menu_bar);
my $file = $menu_bar->cascade(-label => '~File', -tearoff => 0);
my $help = $menu_bar->cascade(-label => '~Help', -tearoff => 0);

$file->command(-label => "Save Tab", -command => \&menu_file_save);
$file->command(-label => "Save All", -command => \&menu_file_save_all);
$file->command(-label => "Exit", -command => \&menu_file_exit);

$help->command(-label => "About");
my $ver_dialog = $main_window->Dialog(-title   => 'About Comicman Config', 
                                      -text    => "Comicman Config\n\nVersion $VERSION\n\nBy: Jay Scott Raymond", 
                                      -buttons => ['OK'], 
                                      -bitmap  => 'info');

my $menu = $help->cget('-menu');
$menu->entryconfigure('About', -command => [$ver_dialog  => 'Show']);
#, -command => \&menu_help_about);

return $menu_bar;
}

#################################################################
# Subroutine for the File->Save of tab 1
# Params: none
# Returns: none
#################################################################
sub save_page1() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "save_page1:got $num_params params, incorrect\n";
}
my $item;
my @user_inf_names;

foreach $item(@list_cur_var) {
	push(@user_inf_names,$inf_comic{$item});
}
save_chosen_list(\@user_inf_names);
}

#################################################################
# Subroutine for the File->Save of tab 2
# Params: none
# Returns: none
#################################################################
sub save_page2() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "save_page2:got $num_params params, incorrect\n";
}
my $user_conf;
my $success;

#set for this session
set_cm_data('debug_level', $current_debug);
set_cm_data('log_setting', $current_log);
set_cm_data('network_timeout', $current_timeout);

# write to user config file
$user_conf = get_cm_data('user_conf_dir') . "/" . get_cm_data('user_conf_file');
if( -e $user_conf ) {
     unlink($user_conf);
}
$success = open(CONF, ">$user_conf" );
if ( $success ) {
	print CONF <<ENDC;
log_setting = $current_log
debug_level = $current_debug
network_timeout = $current_timeout
ENDC
	undef $success;
	close(CONF);
} else {
	debug_print(3, "save_page2: open of $user_conf failed\n");
}
}

#################################################################
# Subroutine for File->SaveTab
# Params: none
# Returns: none
#################################################################
sub menu_file_save() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "menu_file_save:got $num_params params, incorrect\n";
}
my $item;
my @user_inf_names;
my $current_page;

$current_page = $main_widget->raised();

if ($current_page eq "page1") {
	save_page1();
}
if ($current_page eq "page2") {
	save_page2();
}

}

#################################################################
# Subroutine for File->SaveAll
# Params: none
# Returns: none
#################################################################
sub menu_file_save_all() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "menu_file_save_all:got $num_params params, incorrect\n";
}
save_page1();
save_page2();
}

#################################################################
# Subroutine for File->Exit
# Params: none
# Returns: none
#################################################################
sub menu_file_exit() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "menu_file_exit:got $num_params params, incorrect\n";
}
	exit;
}

#################################################################
# Subroutine for the add (>) button
# Params: none
# Returns: none
#################################################################
sub button_add_pressed() {
my $num_params=@_;
if ($num_params != 1 ) {
	die "button_add_pressed:got $num_params params, incorrect\n";
}
my @selections;
my @sorted_select;
my @remove_list;
my $item;
my $item_text;

@selections=listbox_get_selected($listbox_available);
@remove_list=reverse sort { $a <=> $b } @selections;
foreach $item(@selections) {
	$item_text = $listbox_available->get($item);
	$listbox_current->insert('end',$item_text); 
}
# use the reverse sorted list for deletion to avoid index issues
foreach $item(@remove_list) {
	$listbox_available->delete($item);
}

@sorted_select = sort @list_cur_var;
@list_cur_var = @sorted_select;
}

#################################################################
# Subroutine for the add all (>>) button
# Params: none
# Returns: none
#################################################################
sub button_add_all_pressed() {
my $num_params=@_;
if ($num_params != 1 ) {
	die "button_add_all_pressed:got $num_params params, incorrect\n";
}
my $item;
my $item_text;
my @sorted_select;

foreach $item_text (@list_avail_var) {
	$listbox_current->insert('end',$item_text);
}

@sorted_select = sort @list_cur_var;
@list_cur_var = @sorted_select;
@list_avail_var = ();
}

#################################################################
# Subroutine for the remove (<) button
# Params: none
# Returns: none
#################################################################
sub button_remove_pressed() {
my $num_params=@_;
if ($num_params != 1 ) {
	die "button_remove_pressed:got $num_params params, incorrect\n";
}
my @selections;
my @sorted_select;
my @remove_list;
my $item;
my $item_text;

@selections=listbox_get_selected($listbox_current);
@remove_list=reverse sort { $a <=> $b } @selections;
foreach $item(@selections) {
	$item_text = $listbox_current->get($item);
	$listbox_available->insert('end',$item_text); 
}
# use the reverse sorted list for deletion to avoid index issues
foreach $item(@remove_list) {
	$listbox_current->delete($item);
}

@sorted_select = sort @list_avail_var;
@list_avail_var = @sorted_select;
}

#################################################################
# Subroutine for the remove all (<<) button
# Params: none
# Returns: none
#################################################################
sub button_remove_all_pressed() {
my $num_params=@_;
if ($num_params != 1 ) {
	die "button_remove_all_pressed:got $num_params params, incorrect\n";
}
my $item;
my $item_text;
my @sorted_select;

foreach $item_text (@list_cur_var) {
	$listbox_available->insert('end',$item_text);
}

@sorted_select = sort @list_avail_var;
@list_avail_var = @sorted_select;
@list_cur_var = ();
}

#################################################################
# Subroutine to initialize the app
# Params: none
# Returns: none
#################################################################
sub init() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "init:got $num_params params, incorrect\n";
}
my $success;
# setup user directories
$success = make_user_dir();
if ( $success == 1 ) {
	debug_print(3, "init: make_user_dir failed\n");
}
# set up gui containers
$main_window = MainWindow->new;
$main_window->geometry( "500x500" );
$main_widget = $main_window->NoteBook()->pack( -fill=>'both', -expand=>1 );
$page1 = $main_widget->add("page1", -label=>"Comic Select", -state=>"normal");
$page2 = $main_widget->add("page2", -label=>"Settings", -state=>"normal");
$comic_selector = $page1->cmComicSelector();
$settings = $page2->cmSettingsWidget();

$main_window->title("Comicman Config"); 
build_selector_form();
build_settings_form();
my $menu_bar = build_menubar();

} # end init 

#################################################################
# Subroutine to construct the comic selector tab
# Params: none
# Returns: none
#################################################################
sub build_selector_form() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "build_selector_form:got $num_params params, incorrect\n";
}
my @inf_names;
my @sys_inf_names;
my $inf_file;
my @chosen_inf_names;
my $file_path;
my $config;
my %inf_data;
my @sys_comic_titles;
my @comic_titles;

$comic_selector->bind('' => sub {$main_window->destroy});
$comic_selector->pack( -fill=>'both', -expand=>1 );

$comic_selector->Subwidget('AddSelected')->bind('<ButtonRelease-1>' => \&button_add_pressed);
$comic_selector->Subwidget('AddAll')->bind('<ButtonRelease-1>' => \&button_add_all_pressed);
$comic_selector->Subwidget('RemoveSelected')->bind('<ButtonRelease-1>' => \&button_remove_pressed);
$comic_selector->Subwidget('RemoveAll')->bind('<ButtonRelease-1>' => \&button_remove_all_pressed);
$listbox_available = $comic_selector->Subwidget('ListboxAvailable');
$listbox_current = $comic_selector->Subwidget('ListboxCurrent');
tie @list_cur_var, "Tk::Listbox", $listbox_current;
tie @list_avail_var, "Tk::Listbox", $listbox_available;

@inf_names=get_inf_list(get_cm_data('inf_files_dir'));
@sys_inf_names=get_inf_list(get_cm_data('sysinf_files_dir'));
# remove items in userInfList that duplicate sysInfList
foreach $inf_file (@sys_inf_names) {
	@inf_names = grep(!/$inf_file/,@inf_names);
}
@chosen_inf_names=get_chosen_list();
# build title-filename hash
foreach $inf_file (@sys_inf_names) {
	$file_path=get_cm_data('sysinf_files_dir') . "/$inf_file";
	$config = new Config::General((-ConfigFile => $file_path , -AutoLaunder => 1));
	if ( defined($config) ) {
		%inf_data = $config->getall;
		undef $config;
	} else {
		$inf_data{fail}=1;
		debug_print(1, "build_form: new Config failed\n");
	}
	$inf_comic{$inf_data{comicTitle}}=$inf_file;
}
# add user infs to the title-filename hash
foreach $inf_file (@inf_names) {
	$file_path=get_cm_data('inf_files_dir') . "/$inf_file";
	$config = new Config::General((-ConfigFile => $file_path , -AutoLaunder => 1));
	if ( defined($config) ) {
		%inf_data = $config->getall;
		undef $config;
	} else {
		$inf_data{fail}=1;
		debug_print(1, "build_form: new Config failed\n");
	}
	$inf_comic{$inf_data{comicTitle}}=$inf_file;
}
# remove the current comics from the system and user comics
if (scalar(@chosen_inf_names) > 0 ) {
	foreach $inf_file (@chosen_inf_names) {
		@sys_inf_names = grep(!/$inf_file/,@sys_inf_names);
		@inf_names = grep(!/$inf_file/,@inf_names);
	}
}
# get the system infs
foreach $inf_file (@sys_inf_names) {
	$file_path=get_cm_data('sysinf_files_dir') . "/$inf_file";
	$config = new Config::General((-ConfigFile => $file_path , -AutoLaunder => 1));
	if ( defined($config) ) {
		%inf_data = $config->getall;
		undef $config;
	} else {
		$inf_data{fail}=1;
		debug_print(1, "build_form2: new Config failed\n");
	}
	push(@sys_comic_titles,$inf_data{comicTitle});
}
# add any user infs
foreach $inf_file (@inf_names) {
	$file_path=get_cm_data('inf_files_dir') . "/$inf_file";
	$config = new Config::General((-ConfigFile => $file_path , -AutoLaunder => 1));
	if ( defined($config) ) {
		%inf_data = $config->getall;
		undef $config;
	} else {
		$inf_data{fail}=1;
		debug_print(1, "build_form3: new Config failed\n");
	}
	push(@sys_comic_titles,$inf_data{comicTitle});
	
}
if (scalar(@sys_comic_titles) > 0 ) { # populate available listbox
     $listbox_available->insert('end',@sys_comic_titles);
}

# get the user chosen infs
foreach $inf_file (@chosen_inf_names) {
	$file_path=get_cm_data('sysinf_files_dir') . "/$inf_file";
	if (!-e $file_path) {
		$file_path=get_cm_data('inf_files_dir') . "/$inf_file";
		if (!-e $file_path) {
			debug_print(1, "fetch_inf_data: $inf_file not found\n");
			next;
		}
	}
	$config = new Config::General((-ConfigFile => $file_path , -AutoLaunder => 1));
	if ( defined($config) ) {
		%inf_data = $config->getall;
		undef $config;
	} else {
		$inf_data{fail}=1;
		debug_print(1, "build_form4: new Config failed\n");
	}
	push(@comic_titles,$inf_data{comicTitle});
	
}
if (scalar(@comic_titles) > 0 ) {
	$listbox_current->insert('end',@comic_titles);
}
}

#################################################################
# Subroutine to construct the settings tab
# Params: none
# Returns: none
#################################################################
sub build_settings_form() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "build_settings_form:got $num_params params, incorrect\n";
}
my $user_conf;
my $iter;

my $log_range_ref = get_cm_data('log_range');
my @log_range = @$log_range_ref; #(0..1);
my $debug_range_ref = get_cm_data('debug_range');
my @debug_range = @$debug_range_ref; #(0..4);
my $timeout_range_ref = get_cm_data('timeout_range');
my @timeout_range = @$timeout_range_ref; #(5..30);

$settings->bind('' => sub {$main_window->destroy});
$settings->pack( -fill=>'none', -expand=>0 , -anchor=>'nw');

if( init_cm_data()==0) {
	debug_print(1, "cmconfig.pl:build_settings_form:init_cm_data failed\n");
}

foreach $iter (@log_range) {
	$settings->Subwidget('log_option_menu')->addOptions("$iter");
}
foreach $iter (@debug_range) {
	$settings->Subwidget('debug_option_menu')->addOptions("$iter");
}
foreach $iter (@timeout_range) {
	$settings->Subwidget('timeout_option_menu')->addOptions("$iter");
}

$settings->Subwidget('debug_option_menu')->configure(-textvariable=>\$current_debug);
$current_log = get_cm_data('log_setting');

$settings->Subwidget('log_option_menu')->configure(-textvariable=>\$current_log);
$current_debug = get_cm_data('debug_level');

$settings->Subwidget('timeout_option_menu')->configure(-textvariable=>\$current_timeout);
$current_timeout = get_cm_data('network_timeout');

}

######################################################################
# Subroutine the get the selections from a listbox
# Param 0: listbox object
# Returns: array of selection IDs
######################################################################
sub listbox_get_selected($) {
my $num_params=@_;
if ($num_params != 1 ) {
	die "listbox_get_selected:got $num_params params, incorrect\n";
}
my ($list_box)=@_;
my $item;
my $item_text;
my @selected_indeces;
my @selected_items;

@selected_indeces = $list_box->curselection();

return @selected_indeces;
}

#################################################################
# the "main" program
# Params: none
# Returns: none
#################################################################
init();
MainLoop;
