#!/usr/bin/perl -w
#T

=head1 NAME

cminfedit.pl

=cut

=head1 SYNOPSIS

Graphical comicman inf file editor.

=cut

=head1 DESCRIPTION

cminfedit.pl is a graphical (PerlTk) editor for the inf files for the ComicMan webcomics downloader. 

=cut

=head1 AUTHOR

Jay Scott Raymond

=cut

=head1 BUGS

None known as yet.

=cut

=head1 SEE ALSO

inf-doc.txt

=cut

=head1 COPYRIGHT

Copyright 2006, Jay Scott Raymond. All Rights Reserved

This program is free software. You may copy or
redistribute it under the same terms as Perl itself.

=cut

use utf8;
use strict;
use warnings;
use lib '/usr/share/comicman/pm';

use subs qw/build_form build_menubar init/; 
use vars qw/$main_window $main_widget $VERSION $comic_type_var $renaming @arguments $num_args/;

$ENV{'PATH'}="/usr/bin:/bin";
$ENV{SHELL} = '/bin/sh' if exists $ENV{SHELL};
delete @ENV{qw(IFS CDPATH ENV BASH_ENV)};   # Make %ENV safer

package main;

use Tk;
use Tk::widgets qw/Dialog/;
use Tk::FileDialog;
use Comiclib;
use cmInfEdit;

#
# Global variables
#
my $main_window;
my $main_widget;
my $VERSION = "0.0.3";
my $inf_version = "0.0.1";
my @text_widgets=('text_comic_title','text_dat_file','text_page_url','text_comic_url','text_reg_ex','text_referer','text_prefix','text_rename_schema','text_new_name','text_fixup_search','label_fixup_replace');
my @option_widgets=('option_comic_type','option_renaming');
my %inf_data;
my $comic_type_var;
my $renaming;
my @comic_types=('named', 'checksum');
my @renamings=('none', 'prefix', 'full');
my @arguments=@ARGV;
my $num_args=@ARGV;

#################################################################
# Subroutine that builds the menu objects
# Params: none
# Returns: none
#################################################################
sub build_menubar() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "build_menus:got $num_params params, incorrect\n";
}
my $menu_bar = $main_window->Menu;
$main_window->configure(-menu => $menu_bar);
my $file = $menu_bar->cascade(-label => '~File', -tearoff => 0);
my $help = $menu_bar->cascade(-label => '~Help', -tearoff => 0);
$file->command(-label => "New", -command => \&menu_file_new);
$file->command(-label => "Open", -command => \&menu_file_open);
$file->command(-label => "Save", -command => \&menu_file_save);
$file->command(-label => "Exit", -command => \&menu_file_exit);

$help->command(-label => "About");

my $ver_dialog = $main_window->Dialog(-title   => 'About Comicman Inf Editor', 
                                      -text    => "Comicman Inf Editor\n\nVersion $VERSION\n\nBy: Jay Scott Raymond", 
                                      -buttons => ['OK'], 
                                      -bitmap  => 'info');

my $menu = $help->cget('-menu');
$menu->entryconfigure('About', -command => [$ver_dialog  => 'Show']);

return $menu_bar;
}

#################################################################
# Subroutine to initialize the app
# Params: none
# Returns: none
#################################################################
sub init() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "init:got $num_params params, incorrect\n";
}
my $cmd_arg;
my $conf;
my $key;
my $value;
# set up gui containers
$main_window = MainWindow->new;
$main_window->geometry( "500x400" );
$main_widget = $main_window->cmInfEdit()->pack( -fill=>'both', -expand=>1 );
build_form();
my $menu_bar = build_menubar();

# load command line inf, if any
if ( $num_args <= 1 ) {
	foreach $cmd_arg (@arguments) {
		chomp $cmd_arg;
		if ( $cmd_arg=~m%[a-zA-Z0-9\-]*\.inf%) {	
			# arg is a .inf
			if( -e $cmd_arg) {
				$conf = new Config::General((-ConfigFile => $cmd_arg , -AutoLaunder => 1));
				%inf_data=$conf->getall();
				undef $conf;
				while ( ($key,$value) = each %inf_data ) {
					chomp $inf_data{$key};
				}
				set_form_data();
			}
		}
	}
} 
else {
	print("Sorry, I can only load one file at a time.\n");
}

}

#################################################################
# Subroutine callback for the renaming optionMenu
# Params: none
# Returns: none
#################################################################
sub renaming_callback() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "renaming_callback:got $num_params params, incorrect\n";
}
if ($renaming eq 'none' ) {
	$main_widget->Subwidget('text_prefix')->configure(-state=>'disabled');
	$main_widget->Subwidget('text_rename_schema')->configure(-state=>'disabled');
	$main_widget->Subwidget('text_new_name')->configure(-state=>'disabled');
}
if ($renaming eq 'prefix' ) {
	$main_widget->Subwidget('text_prefix')->configure(-state=>'normal');
	$main_widget->Subwidget('text_rename_schema')->configure(-state=>'disabled');
	$main_widget->Subwidget('text_new_name')->configure(-state=>'disabled');
}
if ($renaming eq 'full' ) {
	$main_widget->Subwidget('text_prefix')->configure(-state=>'disabled');
	$main_widget->Subwidget('text_rename_schema')->configure(-state=>'normal');
	$main_widget->Subwidget('text_new_name')->configure(-state=>'normal');
}
}

#################################################################
# Subroutine to construct the gui
# Params: none
# Returns: none
#################################################################
sub build_form() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "build_form:got $num_params params, incorrect\n";
}

my $iter;

$main_widget->Subwidget('text_inf_version')->insert(0,$inf_version);
$main_widget->Subwidget('text_inf_version')->configure(-state=>'disabled');

#init the comic type option menu
$main_widget->Subwidget('option_comic_type')->configure(-variable=>\$comic_type_var);
foreach $iter (@comic_types) {
	$main_widget->Subwidget('option_comic_type')->addOptions("$iter");
}
$main_widget->Subwidget('option_comic_type')->setOption($comic_types[0]);

#init the renaming option menu
$main_widget->Subwidget('option_renaming')->configure(-variable=>\$renaming);

foreach $iter (@renamings) {
	$main_widget->Subwidget('option_renaming')->addOptions("$iter");
}
$main_widget->Subwidget('option_renaming')->setOption($renamings[0]);
$main_widget->Subwidget('option_renaming')->configure(-command=>sub{renaming_callback()});
renaming_callback();
}

#################################################################
# Subroutine to set the form widgets to default values
# Params: none
# Returns: none
#################################################################
sub clear_form() {
my $holder;
my $text_len;
# set text fields
$holder = $main_widget->Subwidget('text_comic_title')->get();
$text_len = length($holder);
$main_widget->Subwidget('text_comic_title')->delete(0,$text_len);

$holder = $main_widget->Subwidget('text_dat_file')->get();
$text_len = length($holder);
$main_widget->Subwidget('text_dat_file')->delete(0,$text_len);

$holder = $main_widget->Subwidget('text_page_url')->get();
$text_len = length($holder);
$main_widget->Subwidget('text_page_url')->delete(0,$text_len);

$holder = $main_widget->Subwidget('text_comic_url')->get();
$text_len = length($holder);
$main_widget->Subwidget('text_comic_url')->delete(0,$text_len);

$holder = $main_widget->Subwidget('text_reg_ex')->get();
$text_len = length($holder);
$main_widget->Subwidget('text_reg_ex')->delete(0,$text_len);

$holder = $main_widget->Subwidget('text_fixup_search')->get();
$text_len = length($holder);
$main_widget->Subwidget('text_fixup_search')->delete(0,$text_len);

$holder = $main_widget->Subwidget('text_fixup_replace')->get();
$text_len = length($holder);
$main_widget->Subwidget('text_fixup_replace')->delete(0,$text_len);

$holder = $main_widget->Subwidget('text_referer')->get();
$text_len = length($holder);
$main_widget->Subwidget('text_referer')->delete(0,$text_len);

# disabled Entry widgets cannot be cleared, so set to normal
$main_widget->Subwidget('text_prefix')->configure(-state=>'normal');
$holder = $main_widget->Subwidget('text_prefix')->get();
$text_len = length($holder);
$main_widget->Subwidget('text_prefix')->delete(0,$text_len);

$main_widget->Subwidget('text_rename_schema')->configure(-state=>'normal');
$holder = $main_widget->Subwidget('text_rename_schema')->get();
$text_len = length($holder);
$main_widget->Subwidget('text_rename_schema')->delete(0,$text_len);

$main_widget->Subwidget('text_new_name')->configure(-state=>'normal');
$holder = $main_widget->Subwidget('text_new_name')->get();
$text_len = length($holder);
$main_widget->Subwidget('text_new_name')->delete(0,$text_len);

# set OptionMenus
$main_widget->Subwidget('option_renaming')->setOption($renamings[0]);
$main_widget->Subwidget('option_comic_type')->setOption($comic_types[0]);
}

############################################################################
# Subroutine to set data in the form widgets
############################################################################
sub set_form_data()
{
my $num_params=@_;
if ($num_params != 0 ) {
	die "set_form_data:got $num_params params, incorrect\n";
}

# required fields
$main_widget->Subwidget('text_comic_title')->insert(0,$inf_data{comicTitle});
$main_widget->Subwidget('text_dat_file')->insert(0,$inf_data{datFile});
$main_widget->Subwidget('text_page_url')->insert(0,$inf_data{pageURL});
$main_widget->Subwidget('text_comic_url')->insert(0,$inf_data{comicURL});
$main_widget->Subwidget('text_reg_ex')->insert(0,$inf_data{regEx});
if( $inf_data{comicType} eq 'named' ) {
	$main_widget->Subwidget('option_comic_type')->setOption($comic_types[0]);
} elsif ( $inf_data{comicType} eq 'checksum' ) {
	$main_widget->Subwidget('option_comic_type')->setOption($comic_types[1]);
} else {
	$main_widget->Subwidget('option_comic_type')->setOption($comic_types[0]);
}

# optional fields

if (exists $inf_data{fixUpSearch} ) {
	$main_widget->Subwidget('text_fixup_search')->insert(0,$inf_data{fixUpSearch});
}
if (exists $inf_data{fixUpReplace} ) {
	$main_widget->Subwidget('text_fixup_replace')->insert(0,$inf_data{fixUpReplace});
}
if (exists $inf_data{referer} ) {
	$main_widget->Subwidget('text_referer')->insert(0,$inf_data{referer});
}

if (exists $inf_data{renameIt} ) {
	if ($inf_data{renameIt} eq 'prefix') {
		$main_widget->Subwidget('option_renaming')->setOption($renamings[1]);
		if (exists $inf_data{prefix} ) {
			$main_widget->Subwidget('text_prefix')->insert(0,$inf_data{prefix});
		}
	} elsif ($inf_data{renameIt} eq 'full' ) {
		$main_widget->Subwidget('option_renaming')->setOption($renamings[2]);
		if (exists $inf_data{nameSchema} ) {
			$main_widget->Subwidget('text_rename_schema')->insert(0,$inf_data{nameSchema});
		}
		if (exists $inf_data{newName} ) {
			$main_widget->Subwidget('text_new_name')->insert(0,$inf_data{newName});
		}
	} 
} else {
	$inf_data{renameIt} = "none";
	$main_widget->Subwidget('option_renaming')->setOption($renamings[0]);
}
}

############################################################################
# Subroutine to retrieve data from the form widgets
############################################################################
sub get_form_data()
{
my $num_params=@_;
if ($num_params != 0 ) {
	die "get_form_data:got $num_params params, incorrect\n";
}
my $text_len;
my $err_dialog;
my $key;
my $value;

# required fields
$inf_data{infVersion} = $main_widget->Subwidget('text_inf_version')->get();
$inf_data{comicTitle} = $main_widget->Subwidget('text_comic_title')->get();
$text_len = length($inf_data{comicTitle});
if ($text_len lt 1) {
	$err_dialog = $main_window->messageBox(-icon => 'question', 
                                               -message => 'Comic Title Required', 
                                               -title => 'Error', 
                                               -type => 'Ok', 
                                               -default => 'Ok');
	return;
}

$inf_data{datFile} = $main_widget->Subwidget('text_dat_file')->get();
$text_len = length($inf_data{datFile});
if ($text_len lt 1) {
	$err_dialog = $main_window->messageBox(-icon => 'question', 
                                               -message => 'Dat File Required', 
                                               -title => 'Error', 
                                               -type => 'Ok', 
                                               -default => 'Ok');
	return;
}

$inf_data{pageURL} = $main_widget->Subwidget('text_page_url')->get();
$text_len = length($inf_data{pageURL});
if ($text_len lt 1) {
	$err_dialog = $main_window->messageBox(-icon => 'question', 
                                               -message => 'Page URL Required', 
                                               -title => 'Error', 
                                               -type => 'Ok', 
                                               -default => 'Ok');
	return;
}

$inf_data{comicURL} = $main_widget->Subwidget('text_comic_url')->get();
$text_len = length($inf_data{comicURL});
if ($text_len lt 1) {
	$err_dialog = $main_window->messageBox(-icon => 'question', 
                                               -message => 'Comic URL Required', 
                                               -title => 'Error', 
                                               -type => 'Ok', 
                                               -default => 'Ok');
	return;
}

$inf_data{regEx} = $main_widget->Subwidget('text_reg_ex')->get();
$text_len = length($inf_data{regEx});
if ($text_len lt 1) {
	$err_dialog = $main_window->messageBox(-icon => 'question', 
                                               -message => 'Reg Ex Required', 
                                               -title => 'Error', 
                                               -type => 'Ok', 
                                               -default => 'Ok');
	return;
}

# comic type option menu
$inf_data{comicType} = $comic_type_var;

# optional fields
$inf_data{fixUpSearch} = $main_widget->Subwidget('text_fixup_search')->get();
$text_len = length($inf_data{fixUpSearch});
if ($text_len lt 1) {
	delete($inf_data{fixUpSearch});
}

$inf_data{fixUpReplace} = $main_widget->Subwidget('text_fixup_replace')->get();
$text_len = length($inf_data{fixUpReplace});
if ($text_len lt 1) {
	delete($inf_data{fixUpReplace});
}

$inf_data{referer} = $main_widget->Subwidget('text_referer')->get();
$text_len = length($inf_data{referer});
if ($text_len lt 1) {
	delete($inf_data{referer});
}


$inf_data{renameIt} = $renaming;
if($renaming eq 'prefix') {
	$inf_data{prefix} = $main_widget->Subwidget('text_prefix')->get();
	$text_len = length($inf_data{prefix});
	if ($text_len lt 1) {
		$err_dialog = $main_window->messageBox(	-icon => 'question', 
							-message => 'Prefix Required', 
                                               		-title => 'Error', 
                                               		-type => 'Ok', 
                                               		-default => 'Ok');
		return;
	}
}
if($renaming eq 'full') {
	$inf_data{nameSchema} = $main_widget->Subwidget('text_rename_schema')->get();
	$text_len = length($inf_data{nameSchema});
	if ($text_len lt 1) {
		$err_dialog = $main_window->messageBox(	-icon => 'question', 
							-message => 'Rename Schema Required', 
                                               		-title => 'Error', 
                                               		-type => 'Ok', 
                                               		-default => 'Ok');
		return;
	}	

	$inf_data{newName} = $main_widget->Subwidget('text_new_name')->get();
	$text_len = length($inf_data{newName});
	if ($text_len lt 1) {
		$err_dialog = $main_window->messageBox(	-icon => 'question', 
							-message => 'New Name Required', 
                                               		-title => 'Error', 
                                               		-type => 'Ok', 
                                               		-default => 'Ok');
		return;
	}
}

$inf_data{lastEdited} = get_cm_data('long_date');
while ( ($key,$value) = each %inf_data ) {
	chomp($inf_data{$key});
	#if length < 2 delete, to remove empty optional fields
#	if (length($value) < 2) {
#		delete $inf_data{$key};
#	}
}

}

#################################################################
# Subroutine for File->Exit
# Params: none
# Returns: none
#################################################################
sub menu_file_exit() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "menu_file_exit:got $num_params params, incorrect\n";
}
	exit;
}

#################################################################
# Subroutine for File->New
# Params: none
# Returns: none
#################################################################
sub menu_file_new() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "menu_file_new:got $num_params params, incorrect\n";
}
clear_form();
undef %inf_data;
}

#################################################################
# Subroutine for File->open
# Params: none
# Returns: none
#################################################################
sub menu_file_open() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "menu_file_open:got $num_params params, incorrect\n";
}
my $conf;
my $file;
#my $open_dialog;
my $key;
my $value;
my $iter;
#my $Horiz;
my $types = [['inf file','.inf']];

#$open_dialog = $main_window->FileDialog(-Title => 'Open', -Create => 0);
#$open_dialog->configure(-FPat => '*inf', -ShowAll => 'NO');
#Qt::FileDialog::getOpenFileName(undef, "*.inf", centralWidget(), "Open");
#$file = $open_dialog->Show();
#if (!defined($file)) {
#	return;
#}

$file = $main_window->getOpenFile(-filetypes => $types,
                                  -defaultextension =>'inf',
                                  -title => 'Open');
if ( $file eq "" ) {
	return;
}
clear_form();
$conf = new Config::General((-ConfigFile => $file , -AutoLaunder => 1));
%inf_data=$conf->getall();
undef $conf;
while ( ($key,$value) = each %inf_data ) {
	chomp $inf_data{$key};
}
set_form_data();
}

#################################################################
# Subroutine for File->open
# Params: none
# Returns: none
#################################################################
sub menu_file_save() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "menu_file_save:got $num_params params, incorrect\n";
}
my $types = [['inf file','.inf']];
my $file;
my $conf;

$file = $main_window->getSaveFile(-filetypes => $types,
                                  -defaultextension =>'inf',
                                  -title => 'Open');
if ( $file eq "" ) {
	return;
}

get_form_data();

$conf = new Config::General(\%inf_data);
$conf->save_file($file);
undef $conf;
}

#################################################################
# the "main" program
# Params: none
# Returns: none
#################################################################
init();
MainLoop;
